#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ShowWelcomeMessageOnStartup_1 = require("./services/ShowWelcomeMessageOnStartup");
const DemoRecordingHelper_1 = require("./services/DemoRecordingHelper");
const DemoPlaybackHelper_1 = require("./services/DemoPlaybackHelper");
const LogHelper_1 = require("./utils/LogHelper");
const ShowHelpMessageWhenAsked_1 = require("./services/ShowHelpMessageWhenAsked");
const path_1 = require("path");
const SubscriberManagerFactory_1 = require("./utils/SubscriberManagerFactory");
const ConfigFactory_1 = require("./utils/ConfigFactory");
const log4js_1 = require("log4js");
const log = LogHelper_1.LogHelper.getLogger('Main');
const waitForUserInputThenExit = () => {
    // https://stackoverflow.com/a/19692588/1687436
    console.log('Press any key to exit');
    process.stdin.setRawMode(true);
    process.stdin.resume();
    process.stdin.on('data', process.exit.bind(process, 0));
};
(async () => {
    LogHelper_1.LogHelper.configure(ConfigFactory_1.ConfigFactory.getConfigInstance().getConfig());
    const subscriberManager = SubscriberManagerFactory_1.SubscriberManagerFactory.getSubscriberManager();
    if (!ConfigFactory_1.ConfigFactory.getConfigInstance().csgoExeExists()) {
        log.fatal(`Couldn't find CS:GO's executable at the path '${path_1.join(ConfigFactory_1.ConfigFactory.getConfigInstance().getConfig().csgo.csgo_demos_folder, "..", "csgo.exe")}'.`);
        log.fatal(`This means that DemoRecordingHelper will be unable to locate the demos folder to check for name conflicts.`);
        throw Error('csgo_demos_folder in config.ini is misconfigured.');
    }
    subscriberManager.subscribe(new ShowWelcomeMessageOnStartup_1.ShowWelcomeMessageOnStartup());
    subscriberManager.subscribe(new ShowHelpMessageWhenAsked_1.ShowHelpMessageWhenAsked());
    subscriberManager.subscribe(new DemoRecordingHelper_1.DemoRecordingHelper());
    subscriberManager.subscribe(new DemoPlaybackHelper_1.DemoPlaybackHelper());
    await subscriberManager.init();
    await subscriberManager.begin();
})().catch(reason => {
    log4js_1.configure({
        appenders: {
            out: { type: 'stdout' },
            app: { type: 'file', filename: 'application.log' }
        },
        categories: { default: { appenders: ['out', 'app'], level: `info` } }
    });
    log.fatal(reason);
    log.fatal('Exited due to a fatal error. Please see above for details.');
    waitForUserInputThenExit();
});
//# sourceMappingURL=Main.js.map