import { ListenerService } from "../ListenerService";
import { Pair } from "../utils/Pair";
export declare class DemoRecordingHelper implements ListenerService {
    static RecordingStartMustBeDelayed: string[];
    private static readonly log;
    private static readonly demoRecordingEndRegExp;
    static readonly BeginRecordingCommand = "dh rec";
    private static readonly recordSplitOrNewDemoRegExp;
    private static readonly resultOfRecordCmdRegExp;
    private static readonly beginRecordingAfterNewRoundRegExp;
    /** A flag that gets switched on when recording begins and off when recording stops. */
    private static currentlyRecording;
    name(): string;
    canHandle(consoleLine: string): boolean;
    handleLine(consoleLine: string): Promise<void>;
    static mostRecentDemoInfoToString(demoName: string, existingDemoInfo: Pair<number, number>): string;
    /**
     * Returns the latest demo that goes by the name provided (typically includes the timestamp like "office-1-8-2021").
     * If this returns [0,0], no demo exists with the provided name. If you receive [1,1], the demo exists and is the
     * only demo with that name (meaning there are no additional parts and no sequels). If this returns [2,1], the demo
     * exists and so does a sequel (most likely named "office-1-8-2021-2"). If this returns [2,2], then the latest
     * demo is "office-1-8-2021-2-pt2".
     *
     * @param demoName the name of the demo to look for (before a "-2" or "-3" or "-pt2" or "-pt3)
     * @returns a Pair with the left number being the highest demo number found with this name and the right number being the highest part number associated with the highest demo number. If one of these values is 0, that respective part of the demo name doesn't exist in the latest demo under the specified demoName.
     */
    static findLatestDemoWithName(demoName: string): Pair<number, number>;
    private promptUserForNewOrSplitDemo;
    private handleStartRecord;
    private applyRecordingPreferences;
    private attemptStartRecording;
    /**
     * This method is a synchronous way to check if DemoRecordingHelper is facilitating the recording of a demo.
     * It does _not_ interact with CS:GO's console meaning it can be used without causing a deadlock in
     * various ListenerService.canHandle() methods.
     * This method was created for the express purpose of preventing DemoPlaybackHelper from immediately
     * unmuting the player when recording began (as the message that DemoPlaybackHelper is meant to read is printed
     * to the console with an echo, intending for it to be read when playing the demo back).
     * @returns whether DemoRecordingHelper is currently managing the recording of a demo
     */
    static synchronouslyCheckIfRecording(): boolean;
}
