"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Config = void 0;
const ini_1 = require("ini");
const fs_1 = require("fs");
const log4js_1 = require("log4js");
const path_1 = require("path");
const os_1 = require("os");
class Config {
    constructor() {
        this.config_path = path_1.join(__dirname, "..", "..", "config.ini");
        this.config_template_path = path_1.join(__dirname, "..", "..", "config.template.ini");
        //TODO: Implement
        // configValid(): boolean {
        //     return true;
        // }
        this.getConfig = () => {
            return this.config;
        };
        this.csgoExeNameForPlatform = () => {
            switch (os_1.type()) {
                case 'Windows_NT':
                    return 'csgo.exe';
                case 'Linux':
                    return 'csgo_linux64';
                case 'Darwin':
                    return 'csgo_osx64';
                default:
                    throw Error(`UNSUPPORTED OS TYPE ${os_1.type()}`);
            }
        };
        if (!this.configExists()) {
            // noinspection SpellCheckingInspection
            log4js_1.configure({
                appenders: {
                    out: { type: 'stdout' },
                    app: { type: 'file', filename: 'application.log' }
                },
                categories: { default: { appenders: ['out', 'app'], level: `info` } }
            });
            const log = log4js_1.getLogger('Config');
            log.fatal(`Couldn't find 'config.ini' expected at path '${this.config_path}'.`);
            log.info(`It seems you haven't made a config.ini file yet. I'll make one for you now...`);
            fs_1.copyFileSync(this.config_template_path, this.config_path, fs_1.constants.COPYFILE_EXCL);
            log.info(`Created 'config.ini' at path ${this.config_path}`);
            log.info(`Please modify config.ini to match your desired settings and then launch this program again.`);
            throw Error(`config.ini not found at path '${this.config_path}'`);
        }
        this.config = ini_1.parse(fs_1.readFileSync(this.config_path, 'utf-8'));
        // if (!this.configValid()) {
        //     //TODO: Check that all settings used by CS:GO Demo Manager are present and valid
        // }
    }
    configExists() {
        return fs_1.existsSync(this.config_path);
    }
    csgoExeExists() {
        return fs_1.existsSync(path_1.join(this.getConfig().csgo.csgo_demos_folder, "..", this.csgoExeNameForPlatform()));
    }
}
exports.Config = Config;
//# sourceMappingURL=Config.js.map