"use strict";
/*
 * A utility for retrieving the values of console variables.
 *
 * Console variables are normally set by naming them and specifying their value like so: "voice_loopback 1".
 * To retrieve the current value of a console variable, you can simply submit its name and the console will return
 * its value. The format is a little loose so this utility provides helper functions for retrieving the values.
 * Here is the output from submitting "game_mode":
 *
 * "game_mode" = "1" ( def. "0" ) game client replicated                            - The current game mode (based on game type). See GameModes.txt.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cvars = void 0;
const LogHelper_1 = require("./LogHelper");
const SubscriberManagerFactory_1 = require("./SubscriberManagerFactory");
class Cvars {
}
exports.Cvars = Cvars;
Cvars.log = LogHelper_1.LogHelper.getLogger('Cvars');
Cvars.getCvar = async (cvarName) => {
    Cvars.log.debug(`Retrieving value of cvar '${cvarName}'...`);
    return await SubscriberManagerFactory_1.SubscriberManagerFactory.getSubscriberManager().requestCvarValue(cvarName);
};
Cvars.setCvar = (cvarName, value) => {
    SubscriberManagerFactory_1.SubscriberManagerFactory.getSubscriberManager().sendMessage(`${cvarName} ${value}`);
    Cvars.log.debug(`Set value of cvar '${cvarName}' to '${value}'.`);
};
//# sourceMappingURL=Cvars.js.map