import { ListenerService } from "../ListenerService";
export declare class SubscriberManager {
    private readonly cvarEchoRegExp;
    private readonly config;
    private readonly port;
    private readonly log;
    private readonly subscriberLog;
    private readonly cvarSubscribersLog;
    private readonly valueListenersLog;
    private initialized;
    private alive;
    private readonly waitOnOpts;
    private socket;
    private reader;
    /** These subscribers are expecting one or more lines of text, unprompted. They're here to stay for the most part. */
    private subscribers;
    /** These subscribers are expecting a single line of output as soon as possible (like a cvar value) */
    private subscribedCvarValues;
    /**
     * These subscribers act much like the cvar subscribers and are removed once dealt with. A regular expression is
     * used to determine if the given console output is meant for the temporary subscriber. If there's a match,
     * the subscriber is removed and the promise is fulfilled with the line output that matched the RegEx.
     */
    private specialOutputGrabbers;
    init: () => Promise<void>;
    begin: () => Promise<void>;
    waitForConsoleSocket: () => Promise<void>;
    /**
     * @returns whether this instance has had init() called on it
     */
    isInitialized: () => boolean;
    /**
     * Determines whether this instance is alive or not. You should always check isInitialized() first.
     * isAlive() will return false if init() hasn't been called and this would mean that this instance ISN'T DEAD
     * but simply has not been initialized yet.
     *
     * @returns whether this instance is still alive or not (has an open socket and is still running the the big for loop)
     * If still running, this will be true. If not started yet, undefined will be returned. If the loop has since stopped,
     * false will be returned.
     */
    isAlive: () => boolean | undefined;
    /**
     * Write one or many commands to the console.
     * @param commandOrArray a single string or an array of strings to be written to the console
     */
    sendMessage: (commandOrArray: string | string[]) => void;
    requestCvarValue: (cvarName: string) => Promise<number>;
    searchForValue: (command: string | string[], regex: RegExp, isUserDecision: boolean, additionalDetailsAboutRequest?: string | undefined) => Promise<string>;
    subscribe: (listener: ListenerService) => void;
    unsubscribe: (listener: ListenerService) => void;
    private logListenerError;
}
