"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.VoicePlayerVolume = void 0;
const pDefer = require("p-defer");
const LogHelper_1 = require("./LogHelper");
const SubscriberManagerFactory_1 = require("./SubscriberManagerFactory");
class VoicePlayerVolume {
}
exports.VoicePlayerVolume = VoicePlayerVolume;
VoicePlayerVolume.log = LogHelper_1.LogHelper.getLogger('VoicePlayerVolume');
VoicePlayerVolume.setVoicePlayerVolumeByName = async (playerName, volume) => {
    let players = await VoicePlayerVolume.getVoicePlayerVolumeValues();
    const player = players.find((value) => value.PlayerName === playerName);
    if (!player) {
        VoicePlayerVolume.log.warn(`Couldn't find player with name '${playerName}' when attempting to set voice volume.`);
    }
    else {
        VoicePlayerVolume.setVoicePlayerVolume(player.PlayerNumber, volume);
    }
};
VoicePlayerVolume.setVoicePlayerVolume = (playerNumber, volume) => {
    SubscriberManagerFactory_1.SubscriberManagerFactory.getSubscriberManager().sendMessage(`voice_player_volume ${playerNumber} ${volume}`);
};
VoicePlayerVolume.getVoicePlayerVolumeValues = async () => {
    const deferred = pDefer();
    const listener = new VoicePlayerVolumeListener(deferred);
    SubscriberManagerFactory_1.SubscriberManagerFactory.getSubscriberManager().subscribe(listener);
    SubscriberManagerFactory_1.SubscriberManagerFactory.getSubscriberManager().sendMessage('voice_player_volume');
    deferred.promise.then(() => SubscriberManagerFactory_1.SubscriberManagerFactory.getSubscriberManager().unsubscribe(listener));
    return deferred.promise;
};
class VoicePlayerVolumeListener {
    constructor(promise) {
        this.players = [];
        this.reading = false;
        this.promise = promise;
    }
    name() {
        return VoicePlayerVolumeListener.name;
    }
    canHandle(consoleLine) {
        return VoicePlayerVolumeListener.isVoicePlayerVolumePadding(consoleLine) || VoicePlayerVolumeListener.voicePlayerVolumeRegExp.test(consoleLine);
    }
    handleLine(consoleLine) {
        if (!this.reading) {
            if (VoicePlayerVolumeListener.paddingDashesRegExp.test(consoleLine)) {
                this.reading = true;
            }
        }
        else {
            if (VoicePlayerVolumeListener.voicePlayerVolumeRegExp.test(consoleLine)) {
                let playerInfo = VoicePlayerVolumeListener.voicePlayerVolumeRegExp.exec(consoleLine);
                const playerVolume = {
                    PlayerNumber: Number(playerInfo[1]),
                    PlayerName: String(playerInfo[2].trim()),
                    Volume: Number(playerInfo[3]),
                };
                this.players.push(playerVolume);
                /*
                 * There is no need to write out "else if (VoicePlayerVolumeListener.paddingDashesRegExp.test(consoleLine)"
                 * here because this line MUST be the padding dashes if it isn't the voicePlayerVolumeRegExp as those
                 * are the only two things that canHandle returns true for. We can simply use a dumb else.
                 */
            }
            else {
                this.reading = false;
                this.promise.resolve(this.players);
            }
        }
        return Promise.resolve(undefined);
    }
}
VoicePlayerVolumeListener.whitespaceRegExp = /\s{2,}/g;
/*
 * TODO: This RegExp could be made a little more robust by matching the line exactly to combat the unlikely event
 *  of another player putting dashes in the chat while an operation is in progress.
 */
VoicePlayerVolumeListener.paddingDashesRegExp = RegExp('-{2,}');
VoicePlayerVolumeListener.voicePlayerVolumeRegExp = RegExp('(\\d+)\ +(.*)\ +(\\d\\.\\d{2})');
VoicePlayerVolumeListener.log = LogHelper_1.LogHelper.getLogger('VoicePlayerVolumeListener');
VoicePlayerVolumeListener.isVoicePlayerVolumePadding = (text) => {
    const processedLine = text.replace(VoicePlayerVolumeListener.whitespaceRegExp, ' ').trim();
    const splitText = processedLine.split(' ');
    let parsedLineIsPaddingCharacters = true;
    for (let element of splitText) {
        if (!VoicePlayerVolumeListener.paddingDashesRegExp.test(element)) {
            parsedLineIsPaddingCharacters = false;
            break;
        }
    }
    return parsedLineIsPaddingCharacters;
};
//# sourceMappingURL=VoicePlayerVolume.js.map